/*! @name videojs-freewheel-ads @version 0.1.2 @license MIT */
import videojs from 'video.js';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

/* global document */
var FwUi = function FwUi(controller) {
  this.controller = controller;
  this.adContainerDiv = document.createElement('div');
  this.controlsDiv = document.createElement('div');
  this.countdownDiv = document.createElement('div');
  this.fullscreenDiv = document.createElement('div');
  this.playPauseDiv = document.createElement('div');
  this.progressDiv = document.createElement('div');
  this.seekBarDiv = document.createElement('div');
  this.sliderDiv = document.createElement('div');
  this.sliderLevelDiv = document.createElement('div');
  this.boundOnMouseUp = this.onMouseUp.bind(this);
  this.boundOnMouseMove = this.onMouseMove.bind(this);
  this.showCountdown = true;

  if (this.controller.getOptions().showCountdown === false) {
    this.showCountdown = false;
  }

  this.createAdContainer();
};

FwUi.prototype.createAdContainer = function () {
  this.assignControlAttributes(this.adContainerDiv, 'fw-ad-container');
  this.adContainerDiv.style.position = 'absolute';
  this.adContainerDiv.style.zIndex = 1111;
  this.adContainerDiv.addEventListener('mouseenter', this.showAdControls.bind(this), false);
  this.adContainerDiv.addEventListener('mouseleave', this.hideAdControls.bind(this), false);
  this.adContainerDiv.addEventListener('click', this.onContainerClicked.bind(this), false);
  this.createControls();
  this.controller.injectAdContainerDiv(this.adContainerDiv);
};

FwUi.prototype.createControls = function () {
  this.assignControlAttributes(this.controlsDiv, 'fw-controls-div');
  this.controlsDiv.style.width = '100%';

  if (!this.controller.getIsMobile()) {
    this.assignControlAttributes(this.countdownDiv, 'fw-countdown-div');
    this.countdownDiv.innerHTML = 'Ad';
    this.countdownDiv.style.display = this.showCountdown ? 'block' : 'none';
  } else {
    this.countdownDiv.style.display = 'none';
  }

  this.assignControlAttributes(this.seekBarDiv, 'fw-seek-bar-div');
  this.seekBarDiv.style.width = '100%';
  this.assignControlAttributes(this.progressDiv, 'fw-progress-div');
  this.assignControlAttributes(this.playPauseDiv, 'fw-play-pause-div');
  this.addClass(this.playPauseDiv, 'fw-playing');
  this.playPauseDiv.addEventListener('click', this.onAdPlayPauseClick.bind(this), false);
  this.assignControlAttributes(this.sliderDiv, 'fw-slider-div');
  this.sliderDiv.addEventListener('mousedown', this.onAdVolumeSliderMouseDown.bind(this), false);
  this.sliderDiv.addEventListener('click', function (e) {
    return e.stopImmediatePropagation();
  }, false); // Hide volume slider controls on iOS as they aren't supported.

  if (this.controller.getIsIos()) {
    this.sliderDiv.style.display = 'none';
  }

  this.assignControlAttributes(this.sliderLevelDiv, 'fw-slider-level-div');
  this.assignControlAttributes(this.fullscreenDiv, 'fw-fullscreen-div');
  this.addClass(this.fullscreenDiv, 'fw-non-fullscreen');
  this.fullscreenDiv.addEventListener('click', this.onAdFullscreenClick.bind(this), false);
  this.adContainerDiv.appendChild(this.controlsDiv);
  this.controlsDiv.appendChild(this.countdownDiv);
  this.controlsDiv.appendChild(this.seekBarDiv);
  this.controlsDiv.appendChild(this.playPauseDiv);
  this.controlsDiv.appendChild(this.sliderDiv);
  this.seekBarDiv.appendChild(this.progressDiv);
  this.sliderDiv.appendChild(this.sliderLevelDiv);
  this.controlsDiv.appendChild(this.fullscreenDiv);
};

FwUi.prototype.onAdPlayPauseClick = function (e) {
  e.stopPropagation();
  this.controller.onAdPlayPauseClick();
};

FwUi.prototype.onAdsPaused = function () {
  this.addClass(this.playPauseDiv, 'fw-paused');
  this.removeClass(this.playPauseDiv, 'fw-playing');
  this.showAdControls();
};

FwUi.prototype.onAdsResumed = function () {
  this.onAdsPlaying();
  this.showAdControls();
};

FwUi.prototype.onAdsPlaying = function () {
  this.addClass(this.playPauseDiv, 'fw-playing');
  this.removeClass(this.playPauseDiv, 'fw-paused');
};

FwUi.prototype.updateFwUi = function (currentTime, remainingTime, duration, adPosition, totalAds) {
  // Update countdown timer data
  var remainingMinutes = Math.floor(remainingTime / 60);
  var remainingSeconds = Math.floor(remainingTime % 60);
  var podCount = ':';

  if (remainingSeconds.toString().length < 2) {
    remainingSeconds = '0' + remainingSeconds;
  }

  if (totalAds > 1) {
    podCount = " (" + adPosition + " of " + totalAds + "):";
  }

  this.countdownDiv.innerHTML = "Ad" + podCount + " " + remainingMinutes + ":" + remainingSeconds; // Update UI

  var playProgressRatio = currentTime / duration;
  var playProgressPercent = playProgressRatio * 100;
  this.progressDiv.style.width = playProgressPercent + '%';
};

FwUi.prototype.onAdVolumeSliderMouseDown = function (e) {
  e.stopPropagation();
  document.addEventListener('mouseup', this.boundOnMouseUp, false);
  document.addEventListener('mousemove', this.boundOnMouseMove, false);
};

FwUi.prototype.onMouseMove = function (e) {
  e.stopPropagation();
  this.changeVolume(e.clientX);
};

FwUi.prototype.onMouseUp = function (e) {
  e.stopPropagation();
  this.changeVolume(e.clientX);
  document.removeEventListener('mouseup', this.boundOnMouseUp);
  document.removeEventListener('mousemove', this.boundOnMouseMove);
};

FwUi.prototype.changeVolume = function (clientX) {
  var percent = (clientX - this.sliderDiv.getBoundingClientRect().left) / this.sliderDiv.offsetWidth;
  percent *= 100; // Bounds value 0-100 if mouse is outside slider region.

  percent = Math.min(Math.max(percent, 0), 100);
  this.sliderLevelDiv.style.width = percent + '%';
  this.controller.setVolume(percent / 100);
};

FwUi.prototype.onAdFullscreenClick = function (e) {
  e.stopPropagation();
  this.controller.toggleFullscreen();
};

FwUi.prototype.onPlayerEnterFullscreen = function () {
  this.addClass(this.fullscreenDiv, 'fw-fullscreen');
  this.removeClass(this.fullscreenDiv, 'fw-non-fullscreen');
};

FwUi.prototype.onPlayerExitFullscreen = function () {
  this.addClass(this.fullscreenDiv, 'fw-non-fullscreen');
  this.removeClass(this.fullscreenDiv, 'fw-fullscreen');
};

FwUi.prototype.onContainerClicked = function (e) {
  e.preventDefault();
  this.controller.onAdClicked(e);
};

FwUi.prototype.showAdContainer = function () {
  this.adContainerDiv.style.display = 'block';
};

FwUi.prototype.hideAdContainer = function () {
  this.adContainerDiv.style.display = 'none';
};

FwUi.prototype.reset = function () {
  this.hideAdContainer();
  this.controlsDiv.style.display = 'none';
  this.countdownDiv.innerHTML = '';
};

FwUi.prototype.onAdError = function () {
  this.hideAdContainer();
};

FwUi.prototype.onAdBreakStart = function () {
  this.showAdContainer();
  this.controlsDiv.style.display = 'block';
  this.onAdsPlaying(); // Start with the ad controls minimized.

  this.hideAdControls();
};

FwUi.prototype.onAdBreakEnd = function () {
  this.hideAdContainer();
  this.controlsDiv.style.display = 'none';
  this.countdownDiv.innerHTML = '';
};

FwUi.prototype.onAllAdsCompleted = function () {
  this.hideAdContainer();
};

FwUi.prototype.onLinearAdStart = function () {
  // Don't bump container when controls are shown
  this.removeClass(this.adContainerDiv, 'bumpable-fw-ad-container');
};

FwUi.prototype.onNonLinearAdLoad = function () {
  // For non-linear ads that show after a linear ad. For linear ads, we show the
  // ad container in onAdBreakStart to prevent blinking in pods.
  this.adContainerDiv.style.display = 'block'; // Bump container when controls are shown

  this.addClass(this.adContainerDiv, 'bumpable-fw-ad-container');
};

FwUi.prototype.onPlayerVolumeChanged = function (volume) {
  this.sliderLevelDiv.style.width = volume * 100 + '%';
};

FwUi.prototype.showAdControls = function () {
  var _this$controller$getO = this.controller.getOptions(),
      disableAdControls = _this$controller$getO.disableAdControls;

  if (!disableAdControls) {
    this.addClass(this.controlsDiv, 'fw-controls-div-showing');
  }
};

FwUi.prototype.hideAdControls = function () {
  this.removeClass(this.controlsDiv, 'fw-controls-div-showing');
};

FwUi.prototype.assignControlAttributes = function (element, controlName) {
  element.id = controlName;
  element.className = controlName;
};

FwUi.prototype.getClassRegexp = function (className) {
  // Matches on
  // (beginning of string OR NOT word char)
  // classname
  // (negative lookahead word char OR end of string)
  return new RegExp('(^|[^A-Za-z-])' + className + '((?![A-Za-z-])|$)', 'gi');
};

FwUi.prototype.addClass = function (element, classToAdd) {
  element.className = element.className.trim() + ' ' + classToAdd;
};

FwUi.prototype.removeClass = function (element, classToRemove) {
  var classRegexp = this.getClassRegexp(classToRemove);
  element.className = element.className.trim().replace(classRegexp, '');
};

var Controller = function Controller(player, options) {
  var _this = this;

  this.player = player;
  this.options = options; // initialize contrib-ads plugin

  var contribAdsDefaults = {
    debug: options.debug
  };

  var adsPluginSettings = _extends({}, contribAdsDefaults, options.contribAdsSettings || {});

  this.player.ads(adsPluginSettings);
  this.adTrackingTimer = null;
  this.boundEndedListener = this.onContentVideoEnded.bind(this);
  this.boundTimeUpdate = this.onContentVideoTimeUpdated.bind(this);
  this.contentPausedOn = 0;
  this.contentSrc = null;
  this.contentSrcType = null;
  this.contentState = '';
  this.currentAdContext = null;
  this.currentAdInstance = null;
  this.currentAdSlot = null;
  this.freewheel = window.tv.freewheel;
  this.fwSDK = window.tv.freewheel.SDK; // have to implement our own tracker since AdManger doesn't seem to dispose/remove listeners properly

  this.isAdPlaying = false;
  this.playerControls = this.player.getChild('controlBar');
  this.setLogLevel(this.options.adManagerLogLevel);
  this.isMobile = navigator.userAgent.match(/iPhone/i) || navigator.userAgent.match(/iPad/i) || navigator.userAgent.match(/Android/i);
  this.isIos = navigator.userAgent.match(/iPhone/i) || navigator.userAgent.match(/iPad/i);
  this.player.ready(function () {
    _this.player.addClass('vjs-freewheel-ads');
  });
  this.player.on('contentchanged', function () {
    _this.fwAdsLog('Video content changed, request ads again');

    _this.requestAds();
  });
  this.player.on('readyforpreroll', function () {
    _this.playback();
  });
  this.player.on('dispose', function () {
    return _this.reset();
  }); // Only one AdManager instance is needed for each player

  this.adManager = new this.fwSDK.AdManager();
  this.adManager.setNetwork(this.options.networkId);
  this.adManager.setServer(this.options.serverURL); // initialize ad ui overlay

  this.FwUi = new FwUi(this); // initialize context events

  this.setupContext(); // start ads request if autoPlay set

  if (this.options.autoPlay) {
    this.requestAds();
  }
};

Controller.prototype.fwAdsLog = function () {
  if (typeof console === 'undefined' || !this.options.debug) {
    return;
  }

  for (var _len = arguments.length, msg = new Array(_len), _key = 0; _key < _len; _key++) {
    msg[_key] = arguments[_key];
  }

  videojs.log.apply(videojs, ['FreewheelAds:'].concat(msg));
};

Controller.prototype.setLogLevel = function (level) {
  if (level === 'quiet') {
    this.fwSDK.setLogLevel(this.fwSDK.LOG_LEVEL_QUIET);
  } else if (level === 'debug') {
    this.fwSDK.setLogLevel(this.fwSDK.LOG_LEVEL_DEBUG);
  } else {
    this.fwSDK.setLogLevel(this.fwSDK.LOG_LEVEL_INFO);
  }
};

Controller.prototype.updateOptions = function (newOptions) {
  this.fwAdsLog('updateOptions, new options: ', newOptions);
  this.options = videojs.mergeOptions(this.options, newOptions); // create context for next request

  this.setupContext();
};

Controller.prototype.getCurrentAdInstance = function () {
  return this.currentAdInstance;
};

Controller.prototype.getCurrentAdSlot = function () {
  return this.currentAdSlot;
};

Controller.prototype.setupContext = function () {
  // set context video info for request
  this.currentAdContext = this.adManager.newContext();
  this.currentAdContext.setProfile(this.options.profileId);
  this.currentAdContext.setSiteSection(this.options.siteSectionId);
  this.currentAdContext.setVideoAsset(this.options.videoAssetId, this.options.videoDuration); // set UI parameters

  this.currentAdContext.setParameter(this.fwSDK.PARAMETER_RENDERER_VIDEO_DISPLAY_CONTROLS_WHEN_PAUSE, false, this.fwSDK.PARAMETER_LEVEL_GLOBAL); // setup listeners

  this.currentAdContext.addEventListener(this.fwSDK.EVENT_SLOT_STARTED, this.onSlotStarted.bind(this));
  this.currentAdContext.addEventListener(this.fwSDK.EVENT_SLOT_ENDED, this.onSlotEnded.bind(this));
  this.currentAdContext.addEventListener(this.fwSDK.EVENT_AD_IMPRESSION, this.onAdStarted.bind(this));
  this.currentAdContext.addEventListener(this.fwSDK.EVENT_AD_IMPRESSION_END, this.onAdEnded.bind(this));
  this.currentAdContext.addEventListener(this.fwSDK.EVENT_ERROR, this.onAdError.bind(this));

  if (!this.isMobile) {
    this.currentAdContext.setParameter(this.fwSDK.PARAMETER_EXTENSION_AD_CONTROL_CLICK_ELEMENT, 'fw-ad-container', this.fwSDK.PARAMETER_LEVEL_GLOBAL);
  }
};

Controller.prototype.requestAds = function () {
  var _this2 = this;

  this.fwAdsLog('Build ad request'); // Configure ad request

  this.prerollSlots = [];
  this.postrollSlots = [];
  this.overlaySlots = [];
  this.midrollSlots = [];
  this.options.adSlots.forEach(function (_ref, index) {
    var id = _ref.id,
        adUnit = _ref.adUnit,
        timePosition = _ref.timePosition;

    _this2.currentAdContext.addTemporalSlot(id, _this2.getSlotType(adUnit), timePosition, null, index + 1);
  }); // Let context object knows where to render the ad

  var displayBaseId = this.player.id() || 'video-player';
  this.currentAdContext.registerVideoDisplayBase(displayBaseId); // Submit ad request
  // Listen to AdManager Events

  this.currentAdContext.addEventListener(this.fwSDK.EVENT_REQUEST_COMPLETE, this.onRequestComplete.bind(this)); // Submit ad request

  this.onAdRequest();
  this.fwAdsLog('Send ad request');
  this.currentAdContext.submitRequest();
};

Controller.prototype.getSlotType = function (adUnit) {
  switch (adUnit) {
    case 'preroll':
      return this.fwSDK.ADUNIT_PREROLL;

    case 'midroll':
      return this.fwSDK.ADUNIT_MIDROLL;

    case 'overlay':
      return this.fwSDK.ADUNIT_OVERLAY;

    case 'postroll':
      return this.fwSDK.ADUNIT_POSTROLL;
  }
}; // Listen for ad request completed and set all slot variables


Controller.prototype.onRequestComplete = function (e) {
  this.fwAdsLog('Ad request completed'); // After request completes, store each roll in corresponding slot array

  if (e.success) {
    var fwTemporalSlots = this.currentAdContext.getTemporalSlots();

    for (var i = 0; i < fwTemporalSlots.length; i += 1) {
      var slot = fwTemporalSlots[i];
      var slotTimePositionClass = slot.getTimePositionClass();

      if (slotTimePositionClass === this.fwSDK.TIME_POSITION_CLASS_PREROLL) {
        this.prerollSlots.push(slot);
      } else if (slotTimePositionClass === this.fwSDK.TIME_POSITION_CLASS_OVERLAY) {
        this.overlaySlots.push(slot);
      } else if (slotTimePositionClass === this.fwSDK.TIME_POSITION_CLASS_MIDROLL) {
        this.midrollSlots.push(slot);
      } else if (slotTimePositionClass === this.fwSDK.TIME_POSITION_CLASS_POSTROLL) {
        this.postrollSlots.push(slot);
      }
    } // tell contril-ads if there is no preroll or postroll


    if (this.prerollSlots.length < 1) {
      this.onNoPreroll();
    }

    if (this.postrollSlots.length < 1) {
      this.onNoPostroll();
    }

    this.onAdsReady();
  }
};

Controller.prototype.playback = function () {
  // Play preroll(s) if a preroll slot exits, otherwise play content
  if (this.prerollSlots.length) {
    this.playPreroll();
  } else {
    this.playContent();
  }
};

Controller.prototype.playPreroll = function () {
  this.fwAdsLog('Play preroll ad'); // Play preroll slot and then remove the played slot from preroll slot array

  if (this.prerollSlots.length) {
    this.onAdBreakStart();
    this.prerollSlots.shift().play();
  } else {
    // When there are no more preroll slots to play, play content
    this.playContent();
  }
};

Controller.prototype.playContent = function () {
  this.onAdBreakEnd(); // Play video content, and add event listener to trigger when video time updates or video content ends

  this.player.ads.contentSrc = this.contentSrc;
  this.player.src({
    src: this.contentSrc,
    type: this.contentSrcType
  });
  this.fwAdsLog('Playing content');
  this.player.on('timeupdate', this.boundTimeUpdate);
  this.contentState = 'VIDEO_STATE_PLAYING';
  this.currentAdContext.setVideoState(this.fwSDK.VIDEO_STATE_PLAYING);
  this.player.play();
};

Controller.prototype.resumeContentAfterMidroll = function () {
  // Resume playing content from when the midroll cue
  this.onAdBreakEnd();
  this.player.ads.contentSrc = this.contentSrc;
  this.player.src({
    src: this.contentSrc,
    type: this.contentSrcType
  });
  this.player.currentTime(this.contentPausedOn);
  this.fwAdsLog("Resume video at: " + this.contentPausedOn);
  this.contentState = 'VIDEO_STATE_PLAYING';
  this.currentAdContext.setVideoState(this.fwSDK.VIDEO_STATE_PLAYING);
  this.player.play();
};

Controller.prototype.playPostroll = function () {
  // Play postroll(s) if exits, otherwise cleanup
  if (this.postrollSlots.length) {
    this.fwAdsLog('Playing postroll');
    this.onAdBreakStart();
    this.postrollSlots.shift().play();
  } else {
    this.fwAdsLog('endlinearmode');
    this.onAdBreakEnd();
    this.reset();
  }
};

Controller.prototype.onSlotStarted = function (e) {
  if (!this.isAdPlaying) {
    return;
  }

  this.currentAdSlot = e.slot;
  this.player.trigger('ads-pod-started', {
    currentSlot: this.currentAdSlot
  });
};

Controller.prototype.onSlotEnded = function (e) {
  if (!this.isAdPlaying) {
    return;
  } // Play the next preroll/postroll ad when either a preroll or postroll stops
  // For a midroll slot, call resumeContentAfterMidroll() and wait for next midroll(if any)


  this.player.trigger('ads-pod-ended', {
    currentSlot: this.currentAdSlot
  });
  this.currentAdSlot = null;
  var slotTimePositionClass = e.slot.getTimePositionClass();

  if (slotTimePositionClass === this.fwSDK.TIME_POSITION_CLASS_PREROLL) {
    this.fwAdsLog('Previous preroll slot ended');
    this.playPreroll();
  } else if (slotTimePositionClass === this.fwSDK.TIME_POSITION_CLASS_POSTROLL) {
    this.fwAdsLog('Previous postroll slot ended');
    this.playPostroll();
  } else if (slotTimePositionClass === this.fwSDK.TIME_POSITION_CLASS_MIDROLL) {
    this.fwAdsLog('Previous midroll slot ended');
    this.resumeContentAfterMidroll();
  }
};

Controller.prototype.onContentVideoTimeUpdated = function () {
  this.fwAdsLog('Video time update, check for midroll/overlay');

  if (this.overlaySlots.length === 0 && this.midrollSlots.length === 0) {
    this.player.off('timeupdate', this.boundTimeUpdate);
  } // Check whether overlay needs to be played


  for (var i = 0; i < this.overlaySlots.length; i += 1) {
    var overlaySlot = this.overlaySlots[i];
    var slotTimePosition = this.overlaySlot.getTimePosition();
    var videoCurrentTime = this.player.currentTime();

    if (Math.abs(videoCurrentTime - slotTimePosition) < 0.5) {
      this.fwAdsLog('Play overlay ad');
      this.overlaySlots.splice(i, 1);
      overlaySlot.play();

      if (document.querySelector('[id^="_fw_ad_container_iframe_Overlay_2"]')) {
        document.querySelector('[id^="_fw_ad_container_iframe_Overlay_1"]').style.marginBottom = '50px';
      }

      return;
    }
  } // Check whether midroll needs to be played


  for (var _i = 0; _i < this.midrollSlots.length; _i += 1) {
    var midrollSlot = this.midrollSlots[_i];

    var _slotTimePosition = midrollSlot.getTimePosition();

    var _videoCurrentTime = this.player.currentTime();

    if (Math.abs(_videoCurrentTime - _slotTimePosition) < 0.5) {
      this.contentPausedOn = this.player.currentTime();
      this.onAdBreakStart();
      this.currentAdContext.setVideoState(this.fwSDK.VIDEO_STATE_PAUSED);
      this.contentState = 'VIDEO_STATE_PAUSED';
      this.midrollSlots.splice(_i, 1);
      this.fwAdsLog('Play midroll ad');
      midrollSlot.play();
      return;
    }
  }
};

Controller.prototype.onContentVideoEnded = function () {
  this.fwAdsLog('Content ended'); // Unbind the event listener for detecting when the content video ends, and play postroll if any

  if (this.contentState === 'VIDEO_STATE_PLAYING') {
    this.player.off('ended', this.onContentVideoEnded.bind(this));
    this.currentAdContext.setVideoState(this.fwSDK.VIDEO_STATE_COMPLETED);
    this.contentState = this.fwSDK.VIDEO_STATE_COMPLETED;

    if (this.postrollSlots.length) {
      this.playPostroll();
    }
  }
};

Controller.prototype.reset = function () {
  this.fwAdsLog('Clean up and reset plugin');
  clearInterval(this.adTrackingTimer);
  this.currentAdInstance = null;
  this.currentAdSlot = null;
  this.contentPausedOn = 0;
  this.contentSrc = null;
  this.contentSrcType = null;
  this.contentState = '';
  this.isAdPlaying = false;
  this.playerControls.show();
  this.FwUi.reset();
  this.player.off('timeupdate', this.boundTimeUpdate);
  this.currentAdContext.removeEventListener(this.fwSDK.EVENT_REQUEST_COMPLETE, this.onRequestComplete.bind(this));
  this.currentAdContext.removeEventListener(this.fwSDK.EVENT_SLOT_STARTED, this.onSlotStarted.bind(this));
  this.currentAdContext.removeEventListener(this.fwSDK.EVENT_SLOT_ENDED, this.onSlotEnded.bind(this));
  this.currentAdContext.removeEventListener(this.fwSDK.EVENT_AD_IMPRESSION, this.onAdStarted.bind(this));
  this.currentAdContext.removeEventListener(this.fwSDK.EVENT_AD_IMPRESSION_END, this.onAdEnded.bind(this));
  this.currentAdContext.removeEventListener(this.fwSDK.EVENT_ERROR, this.onAdError.bind(this));

  if (this.currentAdContext) {
    this.currentAdContext.dispose();
    this.currentAdContext = null;
  }

  if (this.player.ads.inAdBreak()) {
    this.player.ads.disableNextSnapshotRestore = true;
    this.player.ads.endLinearAdMode();
    this.player.trigger('contentresumed');
  }
};

Controller.prototype.onAdsReady = function () {
  this.player.trigger('adsready');
};

Controller.prototype.onNoPreroll = function () {
  this.player.trigger('nopreroll');
};

Controller.prototype.onNoPostroll = function () {
  this.player.trigger('nopostroll');
};

Controller.prototype.onAdClicked = function () {
  if (this.isMobile) {
    // if its mobile, a click on the container should play/pause
    this.onAdPlayPauseClick();
  } else {
    this.player.trigger('ads-click', {
      currentAd: this.currentAdInstance
    });
  }
};

Controller.prototype.onAdRequest = function () {
  this.player.trigger('ads-request');
};

Controller.prototype.onAdStarted = function (e) {
  if (!this.isAdPlaying) {
    return;
  }

  this.currentAdInstance = e.adInstance;
  this.player.trigger('ads-ad-started', {
    currentAd: e.adInstance
  });
};

Controller.prototype.onAdEnded = function (e) {
  if (!this.isAdPlaying) {
    return;
  }

  this.player.trigger('ads-ad-ended', {
    currentAd: e.adInstance
  });
  this.currentAdInstance = null;
};

Controller.prototype.onAdError = function (e) {
  this.player.trigger('ads-error', {
    adError: e
  });
};

Controller.prototype.onFullscreenChange = function () {
  if (this.player.isFullscreen()) {
    this.FwUi.onPlayerEnterFullscreen();
  } else {
    this.FwUi.onPlayerExitFullscreen();
  }
};

Controller.prototype.injectAdContainerDiv = function (adContainerDiv) {
  this.playerControls.el().parentNode.appendChild(adContainerDiv);
};

Controller.prototype.getIsMobile = function () {
  return this.isMobile;
};

Controller.prototype.getIsIos = function () {
  return this.isIos;
};

Controller.prototype.getPlayerId = function () {
  return this.player.id();
};

Controller.prototype.getOptions = function () {
  return this.options;
};

Controller.prototype.getIsMobile = function () {
  return this.isMobile;
};

Controller.prototype.getIsIos = function () {
  return this.isIos;
};

Controller.prototype.setVolume = function (level) {
  return this.player.volume(level);
};

Controller.prototype.toggleFullscreen = function () {
  this.fwAdsLog('Toggle fullscreen');

  if (this.player.isFullscreen()) {
    this.player.exitFullscreen();
    this.FwUi.onPlayerExitFullscreen();
  } else {
    this.player.requestFullscreen();
    this.FwUi.onPlayerEnterFullscreen();
  }
};

Controller.prototype.onAdBreakEnd = function () {
  this.fwAdsLog('Ad break ended');
  this.isAdPlaying = false;
  this.player.on('ended', this.boundEndedListener);

  if (this.player.ads.inAdBreak()) {
    this.player.ads.endLinearAdMode();
  }

  clearInterval(this.adTrackingTimer);
  this.FwUi.onAdBreakEnd();
  this.playerControls.show();
};

Controller.prototype.onAdBreakStart = function () {
  this.fwAdsLog('Starting ad break');
  this.isAdPlaying = true;
  this.contentSrc = this.player.currentSrc();
  this.contentSrcType = this.player.currentType();
  this.player.off('ended', this.boundEndedListener);
  this.player.ads.startLinearAdMode();
  this.playerControls.hide();
  this.player.pause();
  this.FwUi.onAdBreakStart();
  this.adTrackingTimer = setInterval(this.onAdPlayInterval.bind(this), 250);
};

Controller.prototype.onAdPlayInterval = function () {
  if (!this.isAdPlaying) {
    return;
  }

  this.fwAdsLog('Ad playing interval, update UI');
  var duration = this.currentAdSlot.getTotalDuration(); // some ads don't provide this infto so set to zero

  var currentTime = this.currentAdSlot.getPlayheadTime();
  currentTime = currentTime > 0 ? currentTime : 0;
  var remainingTime = duration - currentTime;
  var totalAds = this.currentAdSlot.getAdCount() || 0;
  var adPosition = 1;
  adPosition = this.currentAdSlot.getAdInstances().reduce(function (acc, instance) {
    return acc + (instance._isInitiatedSent ? 1 : 0);
  }, 0);
  this.FwUi.updateFwUi(currentTime, remainingTime, duration, adPosition, totalAds);
};

Controller.prototype.onAdPlayPauseClick = function () {
  if (this.player.ads.inAdBreak() && !this.player.paused()) {
    this.FwUi.onAdsPaused();
    this.fwAdsLog('Ad paused');
    this.player.trigger('ads-pause');
    this.player.pause();
  } else {
    this.FwUi.onAdsPlaying();
    this.fwAdsLog('Ad play');
    this.player.trigger('ads-play');
    this.player.play();
  }
};

var version = "0.1.2";

/* global window */

var defaults = {
  adManagerLogLevel: 'quiet',
  adSlots: [],
  autoPlay: true,
  contribAdsSettings: {},
  debug: false,
  disableAdControls: false,
  networkId: '12345',
  profileId: '12345:profile_id',
  serverURL: 'https://5fd74.v.fwmrm.net/ad/g/1',
  siteSectionId: 'videojs_freewheel_ads',
  videoAssetId: 'videoAssetId',
  videoDuration: 0
}; // Exposed plugin to the player for calling methods

var FreewheelAds = function FreewheelAds(player, options) {
  /**
   * Create a FreewheelAds plugin instance.
   *
   * @param  {Player} player
   *         A Video.js Player instance.
   *
   * @param  {Object} [options]
   *         An optional options object.
   *
   *         While not a core part of the Video.js plugin architecture, a
   *         second argument of options is a convenient way to accept inputs
   *         from your plugin's caller.
   */
  // the parent class will add player under this.player
  var mergedOptions = videojs.mergeOptions(defaults, options); // make sure the AdManager script is attached to the window

  if (!window.tv || !window.tv.freewheel) {
    throw new Error('Missing dependency of AdManager.js. Global object tv doesn\'t exist');
  }

  this.controller = new Controller(player, mergedOptions);

  this.reset = function () {
    return this.controller.reset();
  }.bind(this);

  this.requestAds = function () {
    return this.controller.requestAds();
  }.bind(this);

  this.updateOptions = function (opts) {
    return this.controller.updateOptions(opts);
  }.bind(this);
}; // Include the version number.


FreewheelAds.VERSION = version;

var init = function init(options) {
  this.freewheelAds = new FreewheelAds(this, options);
}; // Register the plugin with video.js.


var registerPlugin = videojs.registerPlugin || videojs.plugin;
registerPlugin('freewheelAds', init);

export default FreewheelAds;
